import threading
import time
from datetime import datetime
from typing import List, Tuple, Dict

from src.api.entities.cliente_aprovado import ApprovedClient
from src.api.entities.cliente_erp_code import ClienteErpCode
from src.jobs.system_jobs import OnlineLogger
from src.api.entities.cliente import Cliente, Endereco
from src.database import utils, queries
from src.database.queries import IntegrationType
import logging
import src.database.connection as database
from src.database.entities.client import Client
from src.entities.log import Log
import src.api.okvendas as api_okvendas
from src.database.utils import DatabaseConfig
from threading import Lock

lock = Lock()
logger = logging.getLogger()
send_log = OnlineLogger.send_log


# region Jobs

def job_send_clients(job_config: dict) -> None:
    with lock:
        """
        Job para enviar os clientes para api okvendas
        Args:
            job_config: Configuração do job obtida na api do oking
        """
        logger.info(job_config.get('job_name') + f' | Executando na Thread {threading.current_thread()}')
        try:
            db_config = utils.get_database_config(job_config)
            db_clients = get_clients(db_config, job_config.get('sql'))

            if len(db_clients) <= 0:
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Nenhum cliente retornado para integracao no momento', 'warning', 'CLIENTE')
                return

            #clients_inserted = insert_out_clients(db_config, db_clients)
            clients_inserted = insert_update_semaphore_client_jobs(job_config, db_config, db_clients)
            if not clients_inserted:
                send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Nao foi possivel inserir os clientes no banco semaforo', 'error', 'CLIENTE')
                return

            clients = [Cliente(
                nome=c.name,
                razao_social=c.company_name,
                sexo='M',
                data_nascimento=None,
                data_bloqueio=c.blocked_date,
                data_constituicao=None,
                cpf=c.cpf or "",
                cnpj=c.cnpj,
                endereco=Endereco(True, c.address_type, c.address, c.zipcode, c.number, c.complement, c.neighbourhood, c.city, c.state, c.residential_phone or c.mobile_phone, str(), c.reference, 'BR', c.ibge_code),
                email=c.email,
                codigo_referencia=c.client_erp,
                telefone_residencial=c.residential_phone,
                telefone_celular=c.mobile_phone,
                inscricao_estadual=c.state_registration,
                compra_liberada=c.purchase_released,
                site_pertencente=c.belonging_site,
                tipo_pessoa=c.person_type,
                limite_credito=c.credit_limit,
                codigo_representante=c.representative_code) for c in db_clients]

            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Enviando clientes via api okvendas', 'info', 'CLIENTE') ###Adicionado###
            total = len(clients)
            page = 10
            limit = 10 if total > 10 else total
            offset = 0

            partial_clients = clients[offset:limit]
            while limit <= total:
                results = api_okvendas.post_clients(partial_clients)
                if len(results) < 1:
                    send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Falha ao integrar clientes: {results[0].message}', 'error', 'CLIENTE')

                for res in results:###Adicionado###
                    identificador = [c.cpf if c.cnpj is None else c.cnpj for c in clients if res.identifiers[0].__contains__(c.cpf or c.cnpj)][0]  ###Adicionado###
                    if res.status == 1:###Adicionado###
                        if protocol_semaphore_send_clients(job_config, db_config, identificador):###Adicionado###
                            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Cliente do cpf/cnpj: {identificador} protocolado no banco semaforo', 'info', 'CLIENTE')###Adicionado###
                        else:###Adicionado###
                            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Falha ao protocolar o cliente do cpf/cnpj: {identificador}', 'warning', 'CLIENTE')###Adicionado###

                #sucessful_results = [result for result in results if result.status == 1] ###Removido###
                failed_results = [result for result in results if result.status > 1]

                #if len(sucessful_results) > 0:
                    #protocol_clients(db_config, [c.codigo_referencia for sr in sucessful_results for c in clients if sr.identifiers[0].__contains__(c.cpf or c.cnpj)])

                if len(failed_results) > 0:
                    for fr in failed_results:
                        for c in partial_clients:
                            time.sleep(0.3)
                            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Falha ao integrar cliente cod.erp {c.codigo_referencia}, cpf/cnpj {c.cnpj if c.cpf is None else c.cpf}: {fr.message}', 'error',
                                          'CLIENTE', c.codigo_referencia)

                limit = limit + page
                offset = offset + page
                partial_clients = clients[offset:limit]

        except Exception as e:
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Erro nao esperado na integracao de clientes: {str(e)}', 'error', 'CLIENTE')


def job_send_approved_clients(job_config: dict) -> None:
    with lock:
        """
        Job para realizar o envio de clientes B2B aprovados para o ERP
    
        Args:
            job_config: Configuração do job obtida na api do oking
        """
        logger.info(job_config.get('job_name') + f' | Executando na Thread {threading.current_thread()}')
        try:
            # 1 Consultar os clientes na api
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Consultando clientes aprovados na api okvendas', 'info', 'CLIENTE')
            clients = api_okvendas.get_approved_clients()
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Encontrado {len(clients.data)} clientes aprovados', 'info', 'CLIENTE')

            # 2 Inserir no banco semaforo
            send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Inserindo clientes aprovados no banco semaforo', 'info', 'CLIENTE')
            db_config = utils.get_database_config(job_config)
            inserted_clients = insert_approved_clients(db_config, clients.data)

            # 3 Disparar nova procedure
            for client in [client_id for client_id, inserted in inserted_clients.items() if inserted]:
                client_data = [c for c in clients.data if c.id == client][0]
                client_erp = None
                try:
                    # 4 Receber codigo erp do cliente da procedure
                    send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Disparando procedure para o cliente {client}', 'info', 'CLIENTE')
                    client_id_sem = get_semaphore_client_id(db_config, client_data)
                    client_erp = run_approved_clients_procedure(db_config, job_config.get('db_seller'), client_id_sem)
                    if client_erp is None:
                        raise 'Codigo erp do cliente nao retornado ou nao foi possivel obter da saida da procedure'

                    # 5 Enviar codigo erp do cliente para api okvendas
                    saved = api_okvendas.put_client_erp_code(ClienteErpCode(client_data.cpf or client_data.cnpj, client_erp))
                    if saved:
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Codigo erp {client_erp} do cliente {client} (id semaforo) salvo com sucesso no okvendas', 'info', 'CLIENTE')
                    else:
                        send_log(job_config.get('job_name'), True, True, f'Nao foi possivel salvar o codigo erp {client_erp} do cliente {client} no okvendas', 'error', 'CLIENTE')
                        continue

                    # 6 Protocolar no semaforo
                    protocoled = protocol_clients(db_config, [client_erp])
                    if protocoled:
                        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Cliente {client} (id semaforo) com codigo erp {client_erp} protocolado com sucesso no banco semaforo', 'info', 'CLIENTE')
                    else:
                        send_log(job_config.get('job_name'), True, True, f'Nao foi possivel protocolar o cliente {client} (id semaforo) com codigo erp {client_erp}', 'error', 'CLIENTE')
                        continue
                except Exception as e:
                    send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f'Erro durante execucao da procedure do cliente {client} (id semaforo): {str(e)}', 'error', 'CLIENTE')

        except Exception as e:
            send_log(job_config.get('job_name'), True, True, f'Erro nao esperado na integracao de clientes aprovados: {str(e)}', 'error', 'CLIENTE')

# endregion Jobs


# region Methods

def run_approved_clients_procedure(db_config: DatabaseConfig, seller_id: int, client_id: int) -> str:
    """
    Dispara a procedure de integracao de clientes aprovados

    Args:
        seller_id: Id da loja
        db_config: Configuracao do banco de dados
        client_id: Id da tabela OPENK_SEMAFORO.CLIENTE

    Returns:
        Codigo do cliente no ERP
    """
    with database.Connection(db_config).get_conect() as conn:
        try:
            with conn.cursor() as cursor:
                if db_config.is_sql_server():
                    cursor.execute('exec openk_semaforo.sp_cliente_aprovado @cliente_id = ?, @loja_id = ?', client_id, seller_id)
                    client_erp_res = cursor.fetchone()
                    if client_erp_res is not None:
                        client_erp, = client_erp_res
                    else:
                        raise 'Nao foi possivel obter o codigo cliente_erp da procedure'
                elif db_config.is_oracle():
                    client_erp_out_value = cursor.var(str)
                    cursor.callproc('OPENK_SEMAFORO.SP_CLIENTE_APROVADO', [client_id, seller_id, client_erp_out_value])
                    client_erp = client_erp_out_value.getvalue()
                elif db_config.is_mysql():
                    result = cursor.callproc('openk_semaforo.SP_CLIENTE_APROVADO', [client_id, seller_id, (0, 'CHAR')])
                    client_erp = result[1]
        except Exception:
            conn.rollback()
            raise

        conn.commit()

    return client_erp


def insert_approved_clients(db_config: DatabaseConfig, clients: List[ApprovedClient]) -> Dict[int, bool]:
    """
    Insere os clientes aprovados no banco semaforo

    Args:
        db_config: Configuracao do banco de dados
        clients: Lista de clientes aprovados vindos da api okvendas

    Returns:
        Quantidade de clientes inseridos
    """
    with database.Connection(db_config).get_conect() as conn:
        results: Dict[int, bool] = {}
        for c in clients:
            with conn.cursor() as cursor:
                existent_client: int = None
                try:
                    existent_client = get_semaphore_client_id(db_config, c)
                except Exception:
                    pass

                address = [a for a in c.addresses if a.id == c.address_id] or c.addresses[0]  # Se nao existir um endereco na lista de enderecos que esteja relacionado com o cliente, pega o primeiro endereco ordenado
                address = address[0] if isinstance(address, list) else address
                if existent_client is None or existent_client <= 0:
                    cursor.execute(queries.get_insert_in_clients(db_config.db_type), queries.get_command_parameter(db_config.db_type, [
                        c.cpf,
                        c.cnpj,
                        datetime.now(),
                        c.name,
                        c.corporate_name,
                        c.cpf,
                        c.cnpj,
                        c.email,
                        c.home_phone,
                        c.mobile_phone,
                        address.zipcode,
                        address.type,
                        address.address,
                        address.number,
                        address.complement,
                        address.neighbourhood,
                        address.city,
                        address.state,
                        address.reference,
                        'IN',  # IN pois esta indo para o ERP
                        datetime.now(),
                        address.ibge_code,
                        c.state_registration
                    ]))
                else:
                    cursor.execute(queries.get_update_in_clients(db_config.db_type), queries.get_command_parameter(db_config.db_type, [datetime.now(), existent_client]))

                results[c.id] = cursor.rowcount > 0

        conn.commit()
        return results


def get_clients(db_config: DatabaseConfig, sql: str) -> List[Client]:
    conn = database.Connection(db_config).get_conect()
    cursor = conn.cursor()

    cursor.execute(sql)
    results = cursor.fetchall()
    columns = [desc[0] for desc in cursor.description]

    cursor.close()
    conn.close()

    clients_list = []
    new: dict = {}
    for row in results:
        for i, c in enumerate(columns):
            new[c.lower()] = row[i]

        clients_list.append(new.copy())

    clients = [Client(**c) for c in clients_list]
    return clients


def insert_out_clients(db_config: DatabaseConfig, clients: List[Client]) -> bool:
    conn = database.Connection(db_config).get_conect()
    cursor = conn.cursor()

    # cursor.executemany(queries.get_insert_out_clients(db_config.db_type), [queries.get_command_parameter(db_config.db_type, [c.client_erp, datetime.now(), datetime.now()]) for c in clients])
    for c in clients:
        cursor.execute(queries.get_insert_out_clients(db_config.db_type), queries.get_command_parameter(db_config.db_type, [c.client_erp, datetime.now(), datetime.now()]))

    result = cursor.rowcount

    cursor.close()
    conn.commit()
    conn.close()

    return result > 0


def protocol_clients(db_config: DatabaseConfig, client_erp_codes: List[str]) -> bool:
    conn = database.Connection(db_config).get_conect()
    cursor = conn.cursor()

    cursor.executemany(queries.get_out_client_protocol_command(db_config.db_type),
                       [queries.get_command_parameter(db_config.db_type, [c]) for c in client_erp_codes])
    result = cursor.rowcount

    cursor.close()
    conn.commit()
    conn.close()

    return result > 0


def insert_update_semaphore_client_jobs(job_config_dict: dict, db_config: DatabaseConfig, lists: List[Client]) -> bool:
    """
    Insere os imposts no banco semáforo
    Args:
        job_config_dict: Configuração do job
        db_config: Configuracao do banco de dados
        lists: Lista de impostos dos produtos

    Returns:
        Boleano indicando se foram inseridos 1 ou mais registros
    """
    params = [(li.cnpj if li.cpf is None else li.cpf, ' ', IntegrationType.CLIENTE.value) for li in lists]

    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        for p in params:
            cursor.execute(queries.get_insert_update_semaphore_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, list(p)))
        cursor.close()
        conn.commit()
        conn.close()
        return cursor.rowcount > 0

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao consultar listas de clientes no banco semaforo: {str(ex)}', 'error', 'CLIENTE')

    return False


def protocol_semaphore_send_clients(job_config: dict, db_config: DatabaseConfig, identifier: str) -> bool:
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    print()
    try:
        if identifier is not None:
            cursor.execute(queries.get_protocol_semaphore_id_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [identifier]))
        count = cursor.rowcount
        cursor.close()
        conn.commit()
        conn.close()
        return count > 0

    except Exception as ex:
        send_log(job_config.get('job_name'), job_config.get('enviar_logs'), False, f' Erro ao protocolar cpf/cnpj do cliente no banco semaforo: {str(ex)}', 'error', 'CLIENTE')



def get_semaphore_client_id(db_config: DatabaseConfig, client_data: ApprovedClient) -> int:
    with database.Connection(db_config).get_conect() as conn:
        cursor = conn.cursor(buffered=True) if db_config.db_type.lower() == 'mysql' else conn.cursor()
        if client_data.cpf is not None and client_data.cpf != '':
            cursor.execute(queries.get_query_client_cpf(db_config.db_type), queries.get_command_parameter(db_config.db_type, [client_data.cpf]))
        elif client_data.cnpj is not None and client_data.cnpj != '':
            cursor.execute(queries.get_query_client_cnpj(db_config.db_type), queries.get_command_parameter(db_config.db_type, [client_data.cnpj]))

        res = cursor.fetchone()
        if res is None:
            raise 'Cliente nao encontrao no banco semaforo'

        cursor.close()
        client_id, = res
        return client_id

# endregion Methods



import threading
from datetime import datetime
import logging
import src.database.connection as database
import src.database.utils as utils
from src.jobs.system_jobs import OnlineLogger
from src.database.utils import DatabaseConfig
import src.api.okvendas as api_okvendas
from src.api import slack
from src.database import queries
import src
import time
from threading import Lock

lock = Lock()
logger = logging.getLogger()
send_log = OnlineLogger.send_log


def job_insert_stock_semaphore(job_config_dict: dict):
    with lock:
        """
        Job para inserir estoques no banco semáforo
        Args:
            job_config_dict: Configuração do job
        """
        logger.info(job_config_dict.get('job_name') + f' | Executando na Thread {threading.current_thread()}')
        db_config = utils.get_database_config(job_config_dict)
        if db_config.sql is None:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                     'Comando sql para inserir estoques no semaforo nao encontrado', 'warning', 'ESTOQUE')
        else:
            db = database.Connection(db_config)
            conn = db.get_conect()
            cursor = conn.cursor()

            try:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                         f'Inserindo estoques no banco semaforo \n {db_config.sql}', 'info', 'ESTOQUE')
                cursor.execute(db_config.sql)
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                         f'{cursor.rowcount} estoques inseridos no banco semaforo', 'info', 'ESTOQUE')
            except Exception as ex:
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                         f'Erro ao inserir estoques no banco semaforo {str(ex)}', 'error', 'ESTOQUE')

            cursor.close()
            conn.commit()
            conn.close()



def job_send_stocks(job_config_dict: dict):
    with lock:
        """
        Job para realizar a atualização dos estoques padrão
        Args:
            job_config_dict: Configuração do job
        """
        logger.info(job_config_dict.get('job_name') + f' | Executando na Thread {threading.current_thread()}')
        db_config = utils.get_database_config(job_config_dict)
        stocks = query_stocks(job_config_dict, db_config)
        atualizados = []
        if len(stocks) or 0 > 0:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                     'Total de estoques a serem atualizados: {len(stocks)}', 'info', '')
            for stock in stocks:
                try:
                    time.sleep(1)
                    response, status_code = api_okvendas.send_stocks(src.client_data.get('url_api') + '/catalogo/estoque',
                                                                     stock, src.client_data.get('token_api'))

                    if response is not None:
                        conexao = database.Connection(db_config)
                        conn = conexao.get_conect()
                        cursor = conn.cursor()

                        codigo_erp = response['codigo_erp']
                        if response['Status'] == 1 or response['Status'] == 'Success':
                            cursor.execute(queries.get_stock_protocol_command(db_config.db_type),
                                           queries.get_command_parameter(db_config.db_type, [codigo_erp]))
                            atualizados.append(response['codigo_erp'])
                        elif (response['Status'] == 'Error' or response['Status'] == 3) and response[
                            'Message'] == "Erro ao atualizar o estoque. Produto não existente":
                            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                                     f'Erro ao atualizar estoque para o sku {codigo_erp}: {response["Message"]}', 'warning',
                                     'ESTOQUE', codigo_erp)
                            cursor.execute(queries.get_stock_protocol_command(db_config.db_type),
                                           queries.get_command_parameter(db_config.db_type, [codigo_erp]))
                        else:
                            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                                     f'Erro ao atualizar estoque para o sku {codigo_erp}: {response["Message"]}', 'warning',
                                     'ESTOQUE', codigo_erp)

                        cursor.close()
                        conn.commit()
                        conn.close()
                except Exception as e:
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                             f'Erro ao atualizar estoque do sku {stock.get("codigo_erp")}: {str(e)}', 'error', 'ESTOQUE',
                             stock.get('codigo_erp'))

            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                     f'Atualizado estoques no semaforo: {len(atualizados) or 0}', 'info', 'ESTOQUE')
        else:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                     f'Nao ha produtos para atualizar estoque no momento', 'warning', 'ESTOQUE')



def job_send_stocks_ud(job_config_dict: dict):
    with lock:
        """
        Job para realizar a atualização dos estoques por unidade de distribuição
        Args:
            job_config_dict: Configuração do job
        """
        logger.info(job_config_dict.get('job_name') + f' | Executando na Thread {threading.current_thread()}')
        db_config = utils.get_database_config(job_config_dict)
        stocks = query_stocks_ud(job_config_dict, db_config)
        p_size = len(stocks) if stocks is not None else 0

        if p_size > 0:
            time.sleep(0.5)
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                     f'Total de produtos para atualizar estoque: {p_size}', 'info', 'ESTOQUE')
            count = 0
            for stock in stocks:
                codigo_erp = stock["codigo_erp"]
                try:
                    response = api_okvendas.send_stocks_ud(
                        src.client_data.get('url_api') + '/catalogo/estoqueUnidadeDistribuicao', [stock],
                        src.client_data.get('token_api'))
                    for res in response:
                        if res.status == 1:
                            protocol_stock(job_config_dict, db_config, codigo_erp)
                            count = count + 1
                        elif (res.status == 'Error' or res.status > 1) and res.message.__contains__("não encontrado"):
                            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                                     f'Erro ao atualizar estoque {codigo_erp} erro da api: {res.message}', 'warning',
                                     'ESTOQUE')
                            protocol_stock(job_config_dict, db_config, codigo_erp)
                        else:
                            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                                     f'Erro ao atualizar estoque do sku {codigo_erp}: {res.message}', 'error', 'ESTOQUE',
                                     codigo_erp)
                except Exception as e:
                    send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True,
                             f'Erro ao atualizar estoque para o sku {stock.get("codigo_erp")}: {str(e)}', 'error',
                             'ESTOQUE', stock.get('codigo_erp'))

            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                     f'Produtos protocolados no semaforo: {count}', 'info', 'ESTOQUE')
        else:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                     f'Nao ha produtos para atualizar estoque', 'warning', 'ESTOQUE')


def query_stocks_ud(job_config_dict: dict, db_config: DatabaseConfig):
    """
    Consulta no banco de dados os estoques pendentes de atualização por unidade de distribuição
    Args:
        job_config_dict: Configuração do job
        db_config: Configuração do banco de dados

    Returns:
    Lista de estoques para realizar a atualização
    """
    stocks = None
    if db_config.sql is None:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                 f'Query estoque de produtos nao configurada', 'warning', 'ESTOQUE')
    else:
        try:
            conexao = database.Connection(db_config)
            conn = conexao.get_conect()
            cursor = conn.cursor()

            # print(db_config.sql)
            cursor.execute(db_config.sql)
            rows = cursor.fetchall()
            # print(rows)
            columns = [col[0] for col in cursor.description]
            results = [dict(zip(columns, row)) for row in rows]

            cursor.close()
            conn.close()
            if len(results) > 0:
                stocks = stock_ud_dict(results)

        except Exception as ex:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'{str(ex)}', 'error',
                     'ESTOQUE')

    return stocks


def query_stocks(job_config_dict: dict, db_config: DatabaseConfig):
    """
    Consulta no banco de dados os estoques pendentes de atualização padrão
    Args:
        job_config_dict: Configuração do job
        db_config: Configuração do banco de dados

    Returns:
    Lista de estoques para realizar a atualização
    """
    produtos = None
    if db_config.sql is None or db_config.sql == '':
        slack.register_warn("Query estoque de produtos nao configurada!")
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                 f'Query estoque de produtos nao configurada', 'warning', 'ESTOQUE')
    else:
        try:
            conexao = database.Connection(db_config)
            conn = conexao.get_conect()
            cursor = conn.cursor()

            # print(db_config.sql)
            cursor.execute(db_config.sql)
            rows = cursor.fetchall()
            # print(rows)
            columns = [col[0] for col in cursor.description]
            results = [dict(zip(columns, row)) for row in rows]

            cursor.close()
            conn.close()
            if len(results) > 0:
                produtos = stock_dict(results)

        except Exception as ex:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'{str(ex)}', 'error',
                     'ESTOQUE')

    return produtos


def stock_dict(produtos):
    lista = []
    for row in produtos:
        pdict = {
            'codigo_erp': str(row['CODIGO_ERP']),
            'quantidade': int(row['QUANTIDADE'])
        }
        lista.append(pdict)

    return lista


def stock_ud_dict(produtos):
    lista = []
    for row in produtos:
        pdict = {
            'unidade_distribuicao': row['CODIGO_UNIDADE_DISTRIBUICAO'],
            'codigo_erp': row['CODIGO_ERP'],
            'quantidade_total': int(row['QUANTIDADE']),
            'parceiro': 1
        }
        lista.append(pdict)

    return lista


def protocol_stock(job_config_dict, db_config: DatabaseConfig, codigo_erp):
    try:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                 f'Protocolando produto {codigo_erp} no semaforo', 'info', 'ESTOQUE')
        conn = database.Connection(db_config).get_conect()
        cursor = conn.cursor()
        cursor.execute(queries.get_stock_protocol_command(db_config.db_type),
                       queries.get_command_parameter(db_config.db_type, [codigo_erp]))
        cursor.close()
        conn.commit()
        conn.close()
    except Exception as e:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False,
                 f'Erro ao protocolar estoque do sku {codigo_erp}: {str(e)}', 'error', '')



from typing import List
from datetime import datetime

class ClientPaymentPlan:
    def __init__(self, codigo_cliente, formas_pagamento):
        self.code_client: str = codigo_cliente
        self.payment_methods: List[str] = formas_pagamento




class ProductTax:
    def __init__(self, codigo_sku, ncm, grupo_tributacao, grupo_cliente,
                 uf_origem, uf_destino, percentual_reducao_base_calcul,
                 mva_iva, icms_intraestadual, icms_interestadual, identificador):
        self.sku_code: str = codigo_sku
        self.ncm: str = ncm
        self.taxation_group: str = grupo_tributacao
        self.customer_group: str = grupo_cliente
        self.origin_uf: str = uf_origem
        self.destination_uf: str = uf_destino
        self.percentage_reduction_base_calculation: float = percentual_reducao_base_calcul
        self.mva_iva: float = mva_iva
        self.intrastate_icms: float = icms_intraestadual
        self.interstate_icms: str = icms_interestadual
        self.identifier: str = identificador



from datetime import datetime

class ClienteErpCode:
    cpf_cnpj: str
    codigo_cliente: str

    def __init__(self, cpf_cnpj: str, codigo_cliente: str) -> None:
        self.cpf_cnpj = cpf_cnpj
        self.codigo_cliente = codigo_cliente


import logging
import threading
from typing import List
import src.database.connection as database
from src.api.entities.representante import Representante, RegiaoVenda
from src.database import queries
from src.database.queries import IntegrationType
from src.database.utils import DatabaseConfig
import src.database.utils as utils
#from src.entities.product import Product
#from src.entities.photos_sku import PhotosSku
import src.api.okvendas as api_okvendas
from src.jobs.system_jobs import OnlineLogger
from src.database.entities.representative import Representative
from threading import Lock

lock = Lock()
logger = logging.getLogger()
send_log = OnlineLogger.send_log


def job_representative(job_config_dict: dict):
    with lock:
        """
        Job para inserir representantes do produto no banco semáforo
        Args:
            job_config_dict: Configuração do job
        """
        logger.info(job_config_dict.get('job_name') + f' | Executando na Thread {threading.current_thread()}')
        db_config = utils.get_database_config(job_config_dict)
        if db_config.sql is None:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Comando sql para inserir os representantes no semaforo nao encontrado', 'warning', 'REPRESENTANTE')
            return

        try:
            db_representative = query_list_representative(job_config_dict, db_config)
            if not insert_update_semaphore_representative(job_config_dict, db_config, db_representative):
                send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Nao foi possivel inserir os representantes no banco semaforo', 'error', 'REPRESENTANTE')
                return

            api_representative = [Representante(
                codigo_externo=r.sku_code,
                nome=r.name,
                telefone_celular=r.mobile_phone,
                login=r.login,
                password=r.password,
                supervisor=r.supervisor,
                ativar_representante=r.representative_active,
                email=r.email,
                regiao_venda=RegiaoVenda(r.rv_name, r.rv_sku_code)) for r in db_representative]



            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Enviando os representantes via api okvendas', 'info', 'REPRESENTANTE')
            total = len(api_representative)
            page = 100
            limit = 100 if total > 100 else total
            offset = 0

            partial_representative = api_representative[offset:limit]
            while limit <= total:
                response = api_okvendas.put_representative(partial_representative)
                for res in response:
                    identificador = [i.codigo_externo for i in api_representative if i.codigo_externo == res.identifiers[0]][0]
                    if res.status == 1:
                        if protocol_semaphore_representative(job_config_dict, db_config, identificador):
                            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Representante {identificador} protocolado no banco semaforo', 'info', 'REPRESENTANTE')
                        else:
                            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f'Falha ao protocolar o representante {identificador}', 'warning', 'REPRESENTANTE')

                limit = limit + page
                offset = offset + page
                partial_representative = api_representative[offset:limit]

        except Exception as ex:
            send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), True, f'Erro {str(ex)}', 'error', 'REPRESENTANTE')


def query_list_representative(job_config_dict: dict, db_config: DatabaseConfig) -> List[Representative]:
    """
        Consultar no banco semáforo os representantes

        Args:
            job_config_dict: Configuração do job
            db_config: Configuração do banco de dados

        Returns:
        Lista de representantes
        """
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        cursor.execute(db_config.sql)
        rows = cursor.fetchall()
        columns = [col[0].lower() for col in cursor.description]
        results = list(dict(zip(columns, row)) for row in rows)
        cursor.close()
        conn.close()
        if len(results) > 0:
            lists = [Representative(**p) for p in results]
            return lists

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao consultar representantes no banco semaforo: {str(ex)}', 'error', 'REPRESENTANTE')

    return []


def insert_update_semaphore_representative(job_config_dict: dict, db_config: DatabaseConfig, lists: List[Representative]) -> bool:
    """
    Insere os representantes no banco semáforo
    Args:
        job_config_dict: Configuração do job
        db_config: Configuracao do banco de dados
        lists: Lista de representantes

    Returns:
        Boleano indicando se foram inseridos 1 ou mais registros
    """
    params = [(li.sku_code, ' ', IntegrationType.REPRESENTANTE.value) for li in lists]

    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        for p in params:
            cursor.execute(queries.get_insert_update_semaphore_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, list(p)))
        cursor.close()
        conn.commit()
        conn.close()
        return cursor.rowcount > 0

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao consultar representantes no banco semaforo: {str(ex)}', 'error', 'REPRESENTANTE')

    return False


def protocol_semaphore_representative(job_config_dict: dict, db_config: DatabaseConfig, identifier: str) -> bool:
    db = database.Connection(db_config)
    conn = db.get_conect()
    cursor = conn.cursor()
    try:
        if identifier is not None:
            cursor.execute(queries.get_protocol_semaphore_id_command(db_config.db_type), queries.get_command_parameter(db_config.db_type, [identifier]))
        count = cursor.rowcount
        cursor.close()
        conn.commit()
        conn.close()
        return count > 0

    except Exception as ex:
        send_log(job_config_dict.get('job_name'), job_config_dict.get('enviar_logs'), False, f' Erro ao protocolar representante no banco semaforo: {str(ex)}', 'error', 'REPRESENTANTE')